const Overview = imports.ui.main.overview;
const GLib = imports.gi.GLib;

const ViewSelector = imports.ui.viewSelector;

let _signal = [];
let _function;

let _manager, _workspace;

var _showAppsButtonChecked;

function hideApps()
{
    if (Overview.viewSelector._showAppsButton.checked)
        Overview.hide();
}

function showApps()
{
    if (_workspace.list_windows().length == 0)
        Overview.viewSelector.showApps();
}

function disconnectWindowSignals()
{
    if (_signal['window-added'])
        _workspace.disconnect(_signal['window-added']);

    if (_signal['window-removed'])
        _workspace.disconnect(_signal['window-removed']);
}

function getWorkspace()
{
    disconnectWindowSignals();

    _workspace = _manager.get_active_workspace();

    _signal['window-added'] = _workspace.connect('window-added', hideApps);
    _signal['window-removed'] = _workspace.connect('window-removed', showApps);
}

function checkWorkspace()
{
    getWorkspace();

    if (!Overview.visible)
        showApps();
    else
        hideApps();
}

function overviewHiding()
{
    // don't show the apps view if we were already looking at it
    if (!_showAppsButtonChecked)
        showApps();
}

function animateFromOverview()
{
    // the original function sets _showAppsButton.checked = false, so we need to copy it to a local variable first
    _showAppsButtonChecked = Overview.viewSelector._showAppsButton.checked;
    _function.apply(this);
}

function init()
{
    _manager = global.screen;
    if (_manager == undefined)
        _manager = global.workspace_manager;
}

function enable()
{
    _signal['workspace-switched'] = _manager.connect('workspace-switched', checkWorkspace);
    _signal['overview-hiding'] = Overview.connect('hiding', overviewHiding);

    _function = ViewSelector.ViewSelector.prototype.animateFromOverview;
    ViewSelector.ViewSelector.prototype.animateFromOverview = animateFromOverview;

    getWorkspace();

    // shows applications at login, but waits for mainloop to turn idle first
    this.event = GLib.idle_add(GLib.PRIORITY_LOW, function()
    {
        showApps();

        return false;
    });
}

function disable()
{
    disconnectWindowSignals();

    Overview.disconnect(_signal['overview-hiding']);
    _manager.disconnect(_signal['workspace-switched']);

    ViewSelector.ViewSelector.prototype.animateFromOverview = _function;
}
